from rest_framework import serializers
from django.db.models import Q
from chat_app.models import ChatRoom
from .models import FollowRequest, Follow, Block, Report
from userprofile_app.models import Profile

class FollowRequestSerializer(serializers.ModelSerializer):
    class Meta:
        model = FollowRequest
        fields = '__all__'

class FollowSerializer(serializers.ModelSerializer):
    class Meta:
        model = Follow
        fields = '__all__'

class BlockSerializer(serializers.ModelSerializer):
    common_reason = serializers.ChoiceField(choices=Block.COMMON_BLOCK_REASONS, required=False)
    reason_details = serializers.CharField(required=False, allow_blank=True)

    class Meta:
        model = Block
        fields = ['blocked', 'common_reason', 'reason_details', 'blocked_at']
        read_only_fields = ['blocked_at']

    def validate_common_reason(self, value):
        if value and value not in dict(Block.COMMON_BLOCK_REASONS).keys():
            raise serializers.ValidationError("Invalid block reason.")
        return value

class ReportSerializer(serializers.ModelSerializer):
    common_reason = serializers.ChoiceField(choices=Report.COMMON_REPORT_REASONS, required=False)
    details = serializers.CharField(required=False, allow_blank=True)

    class Meta:
        model = Report
        fields = ['reported', 'common_reason', 'details', 'reported_at']
        read_only_fields = ['reported_at']

    def validate_common_reason(self, value):
        if value and value not in dict(Report.COMMON_REPORT_REASONS).keys():
            raise serializers.ValidationError("Invalid report reason.")
        return value

class ReasonSerializer(serializers.Serializer):
    code = serializers.CharField()
    label = serializers.CharField()

from django.contrib.auth.models import User  # User model
from userprofile_app.models import Profile  # Assuming a Profile model

class BlockedUserSerializer(serializers.ModelSerializer):
    username = serializers.CharField(source='blocked.username')
    profile_img = serializers.CharField(source='profile.profile_image.url', allow_null=True)  # Assuming a profile model exists

    class Meta:
        model = User
        fields = ['username', 'profile_img']

class BlockListSerializer(serializers.ModelSerializer):
    username = serializers.CharField(source='blocked.username')  # Get the username of the blocked user
    profile_img = serializers.SerializerMethodField()  # Profile image of the blocked user
    blocked_at = serializers.DateTimeField(format="%Y-%m-%dT%H:%M:%SZ")  # Format the datetime for response
    full_name = serializers.CharField(source='blocked.full_name')
    primary_industry = serializers.SerializerMethodField()

    class Meta:
        model = Block
        fields = ['id', 'username', 'profile_img','full_name','primary_industry', 'blocked_at']

    def get_profile_img(self, obj):
        # Retrieve the profile image for the blocked user
        try:
            profile = Profile.objects.get(user=obj.blocked)
            return profile.profile_image.url if profile.profile_image else None
        except Profile.DoesNotExist:
            return None
    def get_primary_industry(self, obj):
        try:
            profile = Profile.objects.get(user=obj.blocked)
            return profile.selected_primary_industry.name if profile.selected_primary_industry else None
        except Profile.DoesNotExist:
            return None
        
class FollowerFollowingSerializer(serializers.ModelSerializer):
    full_name = serializers.SerializerMethodField()
    username = serializers.SerializerMethodField()
    userimage = serializers.ImageField(source='profile_image')
    is_conversation_id_exist = serializers.SerializerMethodField()
    primary_industry = serializers.SerializerMethodField()

    class Meta:
        model = Profile
        fields = ['id','username', 'user_id', 'userimage','full_name','is_conversation_id_exist','primary_industry']
    
    def get_full_name(self, obj):
        return obj.user.full_name
    
    def get_username(self, obj):
        return obj.user.username
    
    def get_is_conversation_id_exist(self, obj):
        request_user = self.context['request'].user  # Logged-in user
        target_user = obj.user  # User being checked

        # Check if a conversation exists between the two users
        is_conversation_id_exist = ChatRoom.objects.filter(
            Q(owner=request_user, members=target_user) | Q(owner=target_user, members=request_user)
        ).exists()

        return is_conversation_id_exist
    def get_primary_industry(self, obj):
        profile = Profile.objects.filter(user_id=obj.user).first()
        if profile:
            return profile.selected_primary_industry.name if profile.selected_primary_industry else None
        return None